﻿using System;
using System.Text;
using scbscript;

namespace example
{
	#region Program

	/// <summary>
	/// Program class
	/// </summary>
	class Program
	{
		/// <summary>
		/// Main function
		/// </summary>
		static void Main(string[] args)
		{
			// Initialize system objects
			scbscript.SystemObjectManager.SetSystemObjectTypes();

			// Add native class type for script
			scbscript.SystemObjectManager.AddClass(typeof(TestValue));
			scbscript.SystemObjectManager.AddClass(typeof(TestObject));
			// Add native global methods class type for script
			scbscript.SystemObjectManager.AddMethods(typeof(TestGlobalMethods));

			// Script
			Script script = new Script(args);

			// Add special object as a string
			script.AddSpecialObject("specialObject", "Special Object");

			// Parse option
			bool isDebug;
			int argNo = 0;
			if(args[argNo] == "-g")
			{
				isDebug = true;
				argNo++;
			} else
			{
				isDebug = false;
			}

			// Set a script source
			if(script.SetScriptCodeFromFile(args[argNo]) > 0)
			{
				foreach (var node in script.Notifies.AllNotifies)
				{
					Console.Error.WriteLine(node.Message);
				}
				End(-1);
			}

			// Compile a script
			if (script.Parse() != 0)
			{
				// Output errors
				foreach (var node in script.Notifies.AllNotifies)
				{
					Console.Error.WriteLine(node.Message);
				}
				End(-1);
			}
			else
			{
				if (isDebug)
				{
					// Run with debugger
					var interpreter = new Interpreter(script, true);
					interpreter.DebuggerPageIcon = "../html/test.png";
					interpreter.DebuggerTitle = "SCBScript Example Debugger";
					interpreter.OnDebuggerClosed = () =>
					{
						Console.WriteLine("<<<< Debugger Closed >>>>");
					};
					interpreter.DebugDelegate = (SyntaxBase synatx, Interpreter inter, SStack[] stacks) =>
					{
					};
					interpreter.DebugOperation = DebugOperation.StepIn;
					if (!interpreter.Run())
					{
						// Output errors
						foreach (var node in script.Notifies.AllNotifies)
						{
							Console.Error.WriteLine(node.Message);
						}
						End(-1);
					}
				}
				else
				{
					// Run
					var interpreter = new Interpreter(script, false);
					if(!interpreter.Run()) {
						// Output errors
						foreach (var node in script.Notifies.AllNotifies)
						{
							Console.Error.WriteLine(node.Message);
						}
						End(-1);
					}
				}
			}
			End(0);
		}

		private static void End(int exitCode)
		{
			System.Environment.Exit(exitCode);
		}
	}

	#endregion

	/// <summary>
	/// Test global methods class
	/// </summary>
	public static class TestGlobalMethods
	{
		public static void PrintHello(Script _script, string arg)
		{
			_script.OutString($" Hello World : [ {arg} ]\n");
		}

		public static string StringMultiply(string str, int count)
		{
			StringBuilder sb = new StringBuilder();
			for(int i = 0; i < count; i++)
			{
				sb.Append(str);
			}
			return sb.ToString();
		}
	}

	/// <summary>
	/// Test value class for script
	/// </summary>
	public class TestValue
	{
		#region Properties

		public int Value1 { get; set; }
		public int Value2 { get; set; }

		#endregion

		#region Constructor

		public TestValue(string value)
		{
			string[] parts = value.Split(',');
			Value1 = int.Parse(parts[0].Trim());
			Value2 = int.Parse(parts[1].Trim());
		}

		#endregion
	}

	/// <summary>
	/// Test object class for script
	/// </summary>
	public class TestObject
	{
		#region Works

		private string _specialObject;

		#endregion

		#region Properties

		public string Value
		{
			get
			{
				return "Value:" + Value1 + "," + Value2;
			}
		}

		public int Value1 { get; set; }
		public int Value2 { get; set; }

		public int this[int index]
		{
			get
			{
				switch (index)
				{
					case 0: return Value1;
					case 1: return Value2;
					default: return -1;
				}
			}
		}

		#endregion

		#region Constructors with special object

		public TestObject(string _specialObject)
		{
			this._specialObject = _specialObject;
			Value1 = 0;
			Value2 = 0;
		}

		// Constructor with special object
		public TestObject(string _specialObject, int value1, int value2)
		{
			this._specialObject = _specialObject;
			Value1 = value1;
			Value2 = value2;
		}

		public TestObject(string _specialObject, TestValue testValue)
		{
			this._specialObject = _specialObject;
			Value1 = testValue.Value1;
			Value2 = testValue.Value2;
		}

		#endregion

		#region Methods

		public int Add()
		{
			return Value1 + Value2;
		}

		public int Mul()
		{
			return Value1 * Value2;
		}

		public TestValue GetTestValue()
		{
			return new TestValue("" + Value1 + ',' + Value2);
		}

		public string GetTestString()
		{
			return _specialObject + ':' + Value1 + ',' + Value2;
		}

		public static void Print(Script _script, TestObject obj)
		{
			_script.OutString("Print : " + obj.Value1 + "," + obj.Value2 + "\n");
		}

		#endregion

		#region Supported operator methods

		public static implicit operator string(TestObject self)
		{
			return self.Value;
		}

		public static implicit operator int(TestObject self)
		{
			return self.Value1 + self.Value2;
		}

		public static TestObject operator+(TestObject s0, TestObject s1)
		{
			return new TestObject(s0._specialObject, s0.Value1 + s1.Value1, s0.Value2 + s1.Value2);
		}

		public static TestObject operator-(TestObject s0, TestObject s1)
		{
			return new TestObject(s0._specialObject, s0.Value1 - s1.Value1, s0.Value2 - s1.Value2);
		}

		public static TestObject operator*(TestObject s0, TestObject s1)
		{
			return new TestObject(s0._specialObject, s0.Value1 * s1.Value1, s0.Value2 * s1.Value2);
		}

		public static TestObject operator/(TestObject s0, TestObject s1)
		{
			return new TestObject(s0._specialObject, s0.Value1 / s1.Value1, s0.Value2 / s1.Value2);
		}

		public static TestObject operator%(TestObject s0, TestObject s1)
		{
			return new TestObject(s0._specialObject, s0.Value1 % s1.Value1, s0.Value2 % s1.Value2);
		}

		public static TestObject operator&(TestObject s0, TestObject s1)
		{
			return new TestObject(s0._specialObject, s0.Value1 & s1.Value1, s0.Value2 & s1.Value2);
		}

		public static TestObject operator|(TestObject s0, TestObject s1)
		{
			return new TestObject(s0._specialObject, s0.Value1 | s1.Value1, s0.Value2 | s1.Value2);
		}

		public static TestObject operator^(TestObject s0, TestObject s1)
		{
			return new TestObject(s0._specialObject, s0.Value1 ^ s1.Value1, s0.Value2 ^ s1.Value2);
		}

		public static TestObject operator -(TestObject s)
		{
			return new TestObject(s._specialObject, -s.Value1, -s.Value2);
		}

		public static TestObject operator ~(TestObject s)
		{
			return new TestObject(s._specialObject, ~s.Value1, ~s.Value2);
		}

		public static bool operator>(TestObject s0, TestObject s1)
		{
			return (s0.Value1 + s1.Value1) > (s0.Value2 + s1.Value2);
		}

		public static bool operator<(TestObject s0, TestObject s1)
		{
			return (s0.Value1 + s1.Value1) < (s0.Value2 + s1.Value2);
		}

		public static bool operator>=(TestObject s0, TestObject s1)
		{
			return (s0.Value1 + s1.Value1) >= (s0.Value2 + s1.Value2);
		}

		public static bool operator<=(TestObject s0, TestObject s1)
		{
			return (s0.Value1 + s1.Value1) <= (s0.Value2 + s1.Value2);
		}

		public static bool operator==(TestObject s0, TestObject s1)
		{
			return s0.Value1 == s1.Value1 && s0.Value2 == s1.Value2;
		}

		public static bool operator!=(TestObject s0, TestObject s1)
		{
			return s0.Value1 != s1.Value1 || s0.Value2 != s1.Value2;
		}

		#endregion

		#region Override

		public override string ToString()
		{
			return Value;
		}

		#endregion
	}
}
